@@ -4,7 +4,7 @@ I/O for AVS-UCD format, cf.
 """
 import logging
 
-import numpy
+import numpy as np
 
 from ..__about__ import __version__ as version
 from .._common import _pick_first_int_data
@@ -51,7 +51,7 @@ def read(filename):
 
 def read_buffer(f):
     # Skip comments and unpack first line
-    num_nodes, num_cells, num_node_data, num_cell_data, _ = numpy.genfromtxt(
+    num_nodes, num_cells, num_node_data, num_cell_data, _ = np.genfromtxt(
         f, max_rows=1, dtype=int, comments="#"
     )
 
@@ -70,15 +70,15 @@ def read_buffer(f):
     # Read cell data
     if num_cell_data:
         cdata = _read_data(f, num_cells, cell_ids)
-        sections = numpy.cumsum([len(c[1]) for c in cells[:-1]])
+        sections = np.cumsum([len(c[1]) for c in cells[:-1]])
         for k, v in cdata.items():
-            cell_data[k] = numpy.split(v, sections)
+            cell_data[k] = np.split(v, sections)
 
     return Mesh(points, cells, point_data=point_data, cell_data=cell_data)
 
 
 def _read_nodes(f, num_nodes):
-    data = numpy.genfromtxt(f, max_rows=num_nodes)
+    data = np.genfromtxt(f, max_rows=num_nodes)
     points_ids = {int(pid): i for i, pid in enumerate(data[:, 0])}
     return points_ids, data[:, 1:]
 
@@ -108,9 +108,9 @@ def _read_cells(f, num_cells, point_ids):
     # Convert to numpy arrays
     for k, c in enumerate(cells):
         cells[k] = CellBlock(
-            c.type, numpy.array(c.data)[:, avsucd_to_meshio_order[c.type]]
+            c.type, np.array(c.data)[:, avsucd_to_meshio_order[c.type]]
         )
-        cell_data["avsucd:material"][k] = numpy.array(cell_data["avsucd:material"][k])
+        cell_data["avsucd:material"][k] = np.array(cell_data["avsucd:material"][k])
     return cell_ids, cells, cell_data
 
 
@@ -124,9 +124,7 @@ def _read_data(f, num_entities, entity_ids):
         line = f.readline().strip().split(",")
         labels[i] = line[0].strip().replace(" ", "_")
         data[labels[i]] = (
-            numpy.empty(num_entities)
-            if dsize == 1
-            else numpy.empty((num_entities, dsize))
+            np.empty(num_entities) if dsize == 1 else np.empty((num_entities, dsize))
         )
 
     for _ in range(num_entities):
@@ -150,23 +148,38 @@ def write(filename, mesh):
             "AVS-UCD requires 3D points, but 2D points given. "
             "Appending 0 third component."
         )
-        mesh.points = numpy.column_stack(
-            [mesh.points[:, 0], mesh.points[:, 1], numpy.zeros(mesh.points.shape[0])]
+        mesh.points = np.column_stack(
+            [mesh.points[:, 0], mesh.points[:, 1], np.zeros(mesh.points.shape[0])]
         )
 
     with open_file(filename, "w") as f:
         # Write meshio version
-        f.write("# Written by meshio v{}\n".format(version))
+        f.write(f"# Written by meshio v{version}\n")
 
         # Write first line
         num_nodes = len(mesh.points)
         num_cells = sum(len(c.data) for c in mesh.cells)
+
+        # Try to find an appropriate materials array
+        key, other = _pick_first_int_data(mesh.cell_data)
+        if key and other:
+            logging.warning(
+                "AVS-UCD can only write one cell data array. "
+                "Picking {}, skipping {}.".format(key, ", ".join(other))
+            )
+        material = (
+            np.concatenate(mesh.cell_data[key])
+            if key
+            else np.zeros(num_cells, dtype=int)
+        )
+
         num_node_data = [
             1 if v.ndim == 1 else v.shape[1] for v in mesh.point_data.values()
         ]
         num_cell_data = [
-            1 if numpy.concatenate(v).ndim == 1 else numpy.concatenate(v).shape[1]
+            1 if np.concatenate(v).ndim == 1 else np.concatenate(v).shape[1]
             for k, v in mesh.cell_data.items()
+            if k != key
         ]
         num_node_data_sum = sum(num_node_data)
         num_cell_data_sum = sum(num_cell_data)
@@ -180,21 +193,21 @@ def write(filename, mesh):
         _write_nodes(f, mesh.points)
 
         # Write cells
-        _write_cells(f, mesh.cells, mesh.cell_data, num_cells)
+        _write_cells(f, mesh.cells, material)
 
         # Write node data
         if num_node_data_sum:
             labels = mesh.point_data.keys()
-            data_array = numpy.column_stack([v for v in mesh.point_data.values()])
+            data_array = np.column_stack([v for v in mesh.point_data.values()])
             _write_data(
                 f, labels, data_array, num_nodes, num_node_data, num_node_data_sum
             )
 
         # Write cell data
         if num_cell_data_sum:
-            labels = list(mesh.cell_data.keys())
-            data_array = numpy.column_stack(
-                [numpy.concatenate(v) for v in mesh.cell_data.values()]
+            labels = [k for k in mesh.cell_data.keys() if k != key]
+            data_array = np.column_stack(
+                [np.concatenate(v) for k, v in mesh.cell_data.items() if k != key]
             )
             _write_data(
                 f, labels, data_array, num_cells, num_cell_data, num_cell_data_sum
@@ -206,19 +219,7 @@ def _write_nodes(f, points):
         f.write("{} {} {} {}\n".format(i + 1, x, y, z))
 
 
-def _write_cells(f, cells, cell_data, num_cells):
-    # try to find an appropriate materials array
-    key, other = _pick_first_int_data(cell_data)
-    if key and other:
-        logging.warning(
-            "AVS-UCD can only write one cell data array. "
-            "Picking {}, skipping {}.".format(key, ", ".join(other))
-        )
-    material = (
-        numpy.concatenate(cell_data[key]) if key else numpy.zeros(num_cells, dtype=int)
-    )
-
-    # Loop over cells
+def _write_cells(f, cells, material):
     i = 0
     for cell_type, v in cells:
         for cell in v[:, meshio_to_avsucd_order[cell_type]]:
@@ -236,10 +237,10 @@ def _write_data(f, labels, data_array, num_entities, num_data, num_data_sum):
     f.write("{} {}\n".format(len(num_data), num_data_str))
 
     for label in labels:
-        f.write("{}, real\n".format(label))
+        f.write(f"{label}, real\n")
 
-    data_array = numpy.column_stack((numpy.arange(1, num_entities + 1), data_array))
-    numpy.savetxt(f, data_array, delimiter=" ", fmt=["%d"] + ["%.14e"] * num_data_sum)
+    data_array = np.column_stack((np.arange(1, num_entities + 1), data_array))
+    np.savetxt(f, data_array, delimiter=" ", fmt=["%d"] + ["%.14e"] * num_data_sum)
 
 
 register("avsucd", [], read, {"avsucd": write})
