@@ -5,12 +5,11 @@ I/O for VTU.
 """
 import base64
 import logging
-import lzma
 import re
 import sys
 import zlib
 
-import numpy
+import numpy as np
 
 from ..__about__ import __version__
 from .._common import (
@@ -25,6 +24,12 @@ from .._exceptions import ReadError
 from .._helpers import register
 from .._mesh import CellBlock, Mesh
 
+# Paraview 5.8.1's built-in Python doesn't have lzma.
+try:
+    import lzma
+except ModuleNotFoundError:
+    lzma = None
+
 
 def num_bytes_to_num_base64_chars(num_bytes):
     # Rounding up in integer division works by double negation since Python
@@ -39,19 +44,36 @@ def _cells_from_data(connectivity, offsets, types, cell_data_raw):
     if len(offsets) != len(types):
         raise ReadError()
 
-    b = numpy.concatenate(
-        [[0], numpy.where(types[:-1] != types[1:])[0] + 1, [len(types)]]
-    )
+    b = np.concatenate([[0], np.where(types[:-1] != types[1:])[0] + 1, [len(types)]])
 
     cells = []
     cell_data = {}
+
     for start, end in zip(b[:-1], b[1:]):
         try:
             meshio_type = vtk_to_meshio_type[types[start]]
         except KeyError:
             raise ReadError("File contains cells that meshio cannot handle.")
-        if meshio_type == "polygon" or meshio_type == "polyhedron":
-            # Polygons and polyhedra have unknown and varying number of nodes per cell.
+
+        # cells with varying number of points
+        special_cells = [
+            "polygon",
+            "VTK_LAGRANGE_CURVE",
+            "VTK_LAGRANGE_TRIANGLE",
+            "VTK_LAGRANGE_QUADRILATERAL",
+            "VTK_LAGRANGE_TETRAHEDRON",
+            "VTK_LAGRANGE_HEXAHEDRON",
+            "VTK_LAGRANGE_WEDGE",
+            "VTK_LAGRANGE_PYRAMID",
+        ]
+        if meshio_type in special_cells:
+            # Polygons have unknown and varying number of nodes per cell.
+            # IMPLEMENTATION NOTE: While polygons are different from other cells
+            # they are much less different than polyhedral cells (for polygons,
+            # it is still possible to define the faces implicitly by assuming that
+            # the cell-nodes have a cyclic ordering). Polygons are therefore
+            # handled here, while the polyhedra have a dedicated function for
+            # processing.
 
             # Index where the previous block of cells stopped. Needed to know the number
             # of nodes for the first cell in the block.
@@ -63,28 +85,29 @@ def _cells_from_data(connectivity, offsets, types, cell_data_raw):
                 first_node = offsets[start - 1]
 
             # Start of the cell-node relation for each cell in this block
-            start_cn = numpy.hstack((first_node, offsets[start:end]))
+            start_cn = np.hstack((first_node, offsets[start:end]))
             # Find the size of each cell
-            size = numpy.diff(start_cn)
+            size = np.diff(start_cn)
 
             # Loop over all cell sizes, find all cells with this size, and assign
             # connectivity
-            for sz in numpy.unique(size):
-                items = numpy.where(size == sz)[0]
-                indices = numpy.add.outer(
+            for sz in np.unique(size):
+                items = np.where(size == sz)[0]
+                indices = np.add.outer(
                     start_cn[items + 1],
                     _vtk_to_meshio_order(types[start], sz, dtype=offsets.dtype) - sz,
                 )
                 cells.append(CellBlock(meshio_type + str(sz), connectivity[indices]))
+
                 # Store cell data for this set of cells
                 for name, d in cell_data_raw.items():
                     if name not in cell_data:
                         cell_data[name] = []
                     cell_data[name].append(d[start + items])
         else:
-            # Same number of nodes per cell
+            # Non-polygonal cell. Same number of nodes per cell makes everything easier.
             n = num_nodes_per_cell[meshio_type]
-            indices = numpy.add.outer(
+            indices = np.add.outer(
                 offsets[start:end],
                 _vtk_to_meshio_order(types[start], n, dtype=offsets.dtype) - n,
             )
@@ -97,18 +120,127 @@ def _cells_from_data(connectivity, offsets, types, cell_data_raw):
     return cells, cell_data
 
 
+def _polyhedron_cells_from_data(offsets, faces, faceoffsets, cell_data_raw):
+    # In general the number of faces will vary between cells, and the
+    # number of nodes vary between faces for each cell. The information
+    # will be stored as a List (one item per cell) of lists (one item
+    # per face of the cell) of np-arrays of node indices.
+
+    cells = {}
+    cell_data = {}
+
+    # The data format for face-cells is:
+    # num_faces_cell_0,
+    #   num_nodes_face_0, node_ind_0, node_ind_1, ..
+    #   num_nodes_face_1, node_ind_0, node_ind_1, ..
+    #   ...
+    # num_faces_cell_1,
+    #   ...
+    # See https://vtk.org/Wiki/VTK/Polyhedron_Support for more.
+
+    # The faceoffsets describes the end of the face description for each
+    # cell. Switch faceoffsets to give start points, not end points
+    faceoffsets = np.append([0], faceoffsets[:-1])
+
+    # Double loop over cells then faces.
+    # This will be slow, but seems necessary to cover all cases
+    for cell_start in faceoffsets:
+        num_faces_this_cell = faces[cell_start]
+        faces_this_cell = []
+        next_face = cell_start + 1
+        for _ in range(num_faces_this_cell):
+            num_nodes_this_face = faces[next_face]
+            faces_this_cell.append(
+                np.array(
+                    faces[next_face + 1 : (next_face + num_nodes_this_face + 1)],
+                    dtype=int,
+                )
+            )
+            # Increase by number of nodes just read, plus the item giving
+            # number of nodes per face
+            next_face += num_nodes_this_face + 1
+
+        # Done with this cell
+        # Find number of nodes for this cell
+        num_nodes_this_cell = np.unique(np.hstack([v for v in faces_this_cell])).size
+
+        key = f"polyhedron{num_nodes_this_cell}"
+        if key not in cells.keys():
+            cells[key] = []
+        cells[key].append(faces_this_cell)
+
+    # The cells will be assigned to blocks according to their number of nodes.
+    # This is potentially a reordering, compared to the ordering in faces.
+    # Cell data must be reorganized accordingly.
+
+    # Start of the cell-node relations
+    start_cn = np.hstack((0, offsets))
+    size = np.diff(start_cn)
+
+    # Loop over all cell sizes, find all cells with this size, and store
+    # cell data.
+    for sz in np.unique(size):
+        # Cells with this number of nodes.
+        items = np.where(size == sz)[0]
+
+        # Store cell data for this set of cells
+        for name, d in cell_data_raw.items():
+            if name not in cell_data:
+                cell_data[name] = []
+            cell_data[name].append(d[items])
+
+    return cells, cell_data
+
+
 def _organize_cells(point_offsets, cells, cell_data_raw):
     if len(point_offsets) != len(cells):
         raise ReadError()
 
     out_cells = []
-    for offset, cls, cdr in zip(point_offsets, cells, cell_data_raw):
-        cls, cell_data = _cells_from_data(
-            cls["connectivity"].ravel(),
-            cls["offsets"].ravel(),
-            cls["types"].ravel(),
-            cdr,
+
+    # IMPLEMENTATION NOTE: The treatment of polyhedral cells is quite a bit different
+    # from the other cells; moreover, there are some strong (?) assumptions on such
+    # cells. The processing of such cells is therefore moved to a dedicated function for
+    # the time being, while all other cell types are treated by the same function.
+    # There are still similarities between processing of polyhedral and the rest, so it
+    # may be possible to unify the implementations at a later stage.
+
+    # Check if polyhedral cells are present.
+    polyhedral_mesh = False
+    for c in cells:
+        if np.any(c["types"] == 42):  # vtk type 42 is polyhedral
+            polyhedral_mesh = True
+            break
+
+    if polyhedral_mesh:
+        # The current implementation assumes a single set of cells, and cannot mix
+        # polyhedral cells with other cell types. It may be possible to do away with
+        # these limitations, but for the moment, this is what is available.
+        if len(cells) > 1:
+            raise ValueError("Implementation assumes single set of cells")
+        if np.any(cells[0]["types"] != 42):
+            raise ValueError("Cannot handle combinations of polyhedra with other cells")
+
+        # Polyhedra are specified by their faces and faceoffsets; see the function
+        # _polyhedron_cells_from_data for more information.
+        faces = cells[0]["faces"]
+        faceoffsets = cells[0]["faceoffsets"]
+        cls, cell_data = _polyhedron_cells_from_data(
+            cells[0]["offsets"], faces, faceoffsets, cell_data_raw[0]
         )
+        # Organize polyhedra in cell blocks according to the number of nodes per cell.
+        for tp, c in cls.items():
+            out_cells.append(CellBlock(tp, c))
+
+    else:
+        for offset, cls, cdr in zip(point_offsets, cells, cell_data_raw):
+            cls, cell_data = _cells_from_data(
+                cls["connectivity"].ravel(),
+                cls["offsets"].ravel(),
+                cls["types"].ravel(),
+                cdr,
+            )
+
         for c in cls:
             out_cells.append(CellBlock(c.type, c.data + offset))
 
@@ -125,7 +257,7 @@ def get_grid(root):
             grid = c
         else:
             if c.tag != "AppendedData":
-                raise ReadError("Unknown main tag '{}'.".format(c.tag))
+                raise ReadError(f"Unknown main tag '{c.tag}'.")
             if appended_data is not None:
                 raise ReadError("More than one AppendedData section found.")
             if c.attrib["encoding"] != "base64":
@@ -168,10 +300,9 @@ def _parse_raw_binary(filename):
     appended_data_tag = root.find("AppendedData")
     appended_data_tag.set("encoding", "base64")
 
-    if "compressor" in root.attrib:
-        c = {"vtkLZMADataCompressor": lzma, "vtkZLibDataCompressor": zlib}[
-            root.get("compressor")
-        ]
+    compressor = root.get("compressor")
+    if compressor is not None:
+        c = {"vtkLZMADataCompressor": lzma, "vtkZLibDataCompressor": zlib}[compressor]
         root.attrib.pop("compressor")
 
         # raise ReadError("Compressed raw binary VTU files not supported.")
@@ -181,10 +312,10 @@ def _parse_raw_binary(filename):
             da_tag = root.find(".//DataArray[@offset='%d']" % i)
             da_tag.set("offset", "%d" % len(arrays))
 
-            num_blocks = int(numpy.frombuffer(data[i : i + dtype.itemsize], dtype)[0])
+            num_blocks = int(np.frombuffer(data[i : i + dtype.itemsize], dtype)[0])
             num_header_items = 3 + num_blocks
             num_header_bytes = num_header_items * dtype.itemsize
-            header = numpy.frombuffer(data[i : i + num_header_bytes], dtype)
+            header = np.frombuffer(data[i : i + num_header_bytes], dtype)
 
             block_data = b""
             j = 0
@@ -197,7 +328,7 @@ def _parse_raw_binary(filename):
                 )
                 j += block_size
 
-            block_size = numpy.array([len(block_data)]).astype(dtype).tobytes()
+            block_size = np.array([len(block_data)]).astype(dtype).tobytes()
             arrays += base64.b64encode(block_size + block_data).decode()
 
             i += j + num_header_bytes
@@ -209,7 +340,7 @@ def _parse_raw_binary(filename):
             da_tag = root.find(".//DataArray[@offset='%d']" % i)
             da_tag.set("offset", "%d" % len(arrays))
 
-            block_size = int(numpy.frombuffer(data[i : i + dtype.itemsize], dtype)[0])
+            block_size = int(np.frombuffer(data[i : i + dtype.itemsize], dtype)[0])
             arrays += base64.b64encode(
                 data[i : i + block_size + dtype.itemsize]
             ).decode()
@@ -220,16 +351,16 @@ def _parse_raw_binary(filename):
 
 
 vtu_to_numpy_type = {
-    "Float32": numpy.dtype(numpy.float32),
-    "Float64": numpy.dtype(numpy.float64),
-    "Int8": numpy.dtype(numpy.int8),
-    "Int16": numpy.dtype(numpy.int16),
-    "Int32": numpy.dtype(numpy.int32),
-    "Int64": numpy.dtype(numpy.int64),
-    "UInt8": numpy.dtype(numpy.uint8),
-    "UInt16": numpy.dtype(numpy.uint16),
-    "UInt32": numpy.dtype(numpy.uint32),
-    "UInt64": numpy.dtype(numpy.uint64),
+    "Float32": np.dtype(np.float32),
+    "Float64": np.dtype(np.float64),
+    "Int8": np.dtype(np.int8),
+    "Int16": np.dtype(np.int16),
+    "Int32": np.dtype(np.int32),
+    "Int64": np.dtype(np.int64),
+    "UInt8": np.dtype(np.uint8),
+    "UInt16": np.dtype(np.uint16),
+    "UInt32": np.dtype(np.uint32),
+    "UInt64": np.dtype(np.uint64),
 }
 numpy_to_vtu_type = {v: k for k, v in vtu_to_numpy_type.items()}
 
@@ -279,7 +410,7 @@ class VtuReader:
         try:
             self.byte_order = root.attrib["byte_order"]
             if self.byte_order not in ["LittleEndian", "BigEndian"]:
-                raise ReadError("Unknown byte order '{}'.".format(self.byte_order))
+                raise ReadError(f"Unknown byte order '{self.byte_order}'.")
         except KeyError:
             self.byte_order = None
 
@@ -295,7 +426,7 @@ class VtuReader:
                 for data_array in c:
                     field_data[data_array.attrib["Name"]] = self.read_data(data_array)
             else:
-                raise ReadError("Unknown grid subtag '{}'.".format(c.tag))
+                raise ReadError(f"Unknown grid subtag '{c.tag}'.")
 
         if not pieces:
             raise ReadError("No Piece found.")
@@ -359,7 +490,7 @@ class VtuReader:
 
                     cell_data_raw.append(piece_cell_data_raw)
                 else:
-                    raise ReadError("Unknown tag '{}'.".format(child.tag))
+                    raise ReadError(f"Unknown tag '{child.tag}'.")
 
         if not cell_data_raw:
             cell_data_raw = [{}] * len(cells)
@@ -367,16 +498,16 @@ class VtuReader:
         if len(cell_data_raw) != len(cells):
             raise ReadError()
 
-        point_offsets = numpy.cumsum([0] + [pts.shape[0] for pts in points][:-1])
+        point_offsets = np.cumsum([0] + [pts.shape[0] for pts in points][:-1])
 
         # Now merge across pieces
         if not points:
             raise ReadError()
-        self.points = numpy.concatenate(points)
+        self.points = np.concatenate(points)
 
         if point_data:
             self.point_data = {
-                key: numpy.concatenate([pd[key] for pd in point_data])
+                key: np.concatenate([pd[key] for pd in point_data])
                 for key in point_data[0]
             }
         else:
@@ -387,17 +518,17 @@ class VtuReader:
         )
         self.field_data = field_data
 
-    def read_uncompressed_binary(self, data, data_type):
+    def read_uncompressed_binary(self, data, dtype):
         byte_string = base64.b64decode(data)
 
-        dtype = vtu_to_numpy_type[self.header_type]
+        header_dtype = vtu_to_numpy_type[self.header_type]
         if self.byte_order is not None:
-            dtype = dtype.newbyteorder(
+            header_dtype = header_dtype.newbyteorder(
                 "<" if self.byte_order == "LittleEndian" else ">"
             )
-        num_bytes_per_item = numpy.dtype(dtype).itemsize
+        num_bytes_per_item = np.dtype(header_dtype).itemsize
         total_num_bytes = int(
-            numpy.frombuffer(byte_string[:num_bytes_per_item], dtype)[0]
+            np.frombuffer(byte_string[:num_bytes_per_item], header_dtype)[0]
         )
 
         # Check if block size was decoded separately
@@ -409,31 +540,30 @@ class VtuReader:
             byte_string = byte_string[num_bytes_per_item:]
 
         # Read the block data; multiple blocks possible here?
-        dtype = vtu_to_numpy_type[data_type]
         if self.byte_order is not None:
             dtype = dtype.newbyteorder(
                 "<" if self.byte_order == "LittleEndian" else ">"
             )
-        return numpy.frombuffer(byte_string[:total_num_bytes], dtype=dtype)
+        return np.frombuffer(byte_string[:total_num_bytes], dtype=dtype)
 
-    def read_compressed_binary(self, data, data_type):
+    def read_compressed_binary(self, data, dtype):
         # first read the block size; it determines the size of the header
-        dtype = vtu_to_numpy_type[self.header_type]
+        header_dtype = vtu_to_numpy_type[self.header_type]
         if self.byte_order is not None:
-            dtype = dtype.newbyteorder(
+            header_dtype = header_dtype.newbyteorder(
                 "<" if self.byte_order == "LittleEndian" else ">"
             )
-        num_bytes_per_item = numpy.dtype(dtype).itemsize
+        num_bytes_per_item = np.dtype(header_dtype).itemsize
         num_chars = num_bytes_to_num_base64_chars(num_bytes_per_item)
         byte_string = base64.b64decode(data[:num_chars])[:num_bytes_per_item]
-        num_blocks = numpy.frombuffer(byte_string, dtype)[0]
+        num_blocks = np.frombuffer(byte_string, header_dtype)[0]
 
         # read the entire header
         num_header_items = 3 + int(num_blocks)
         num_header_bytes = num_bytes_per_item * num_header_items
         num_header_chars = num_bytes_to_num_base64_chars(num_header_bytes)
         byte_string = base64.b64decode(data[:num_header_chars])
-        header = numpy.frombuffer(byte_string, dtype)
+        header = np.frombuffer(byte_string, header_dtype)
 
         # num_blocks = header[0]
         # max_uncompressed_block_size = header[1]
@@ -442,25 +572,24 @@ class VtuReader:
 
         # Read the block data
         byte_array = base64.b64decode(data[num_header_chars:])
-        dtype = vtu_to_numpy_type[data_type]
         if self.byte_order is not None:
             dtype = dtype.newbyteorder(
                 "<" if self.byte_order == "LittleEndian" else ">"
             )
-        num_bytes_per_item = numpy.dtype(dtype).itemsize
 
-        byte_offsets = numpy.empty(block_sizes.shape[0] + 1, dtype=block_sizes.dtype)
+        byte_offsets = np.empty(block_sizes.shape[0] + 1, dtype=block_sizes.dtype)
         byte_offsets[0] = 0
-        numpy.cumsum(block_sizes, out=byte_offsets[1:])
+        np.cumsum(block_sizes, out=byte_offsets[1:])
 
+        assert self.compression is not None
         c = {"vtkLZMADataCompressor": lzma, "vtkZLibDataCompressor": zlib}[
             self.compression
         ]
 
         # process the compressed data
-        block_data = numpy.concatenate(
+        block_data = np.concatenate(
             [
-                numpy.frombuffer(
+                np.frombuffer(
                     c.decompress(byte_array[byte_offsets[k] : byte_offsets[k + 1]]),
                     dtype=dtype,
                 )
@@ -473,18 +602,22 @@ class VtuReader:
     def read_data(self, c):
         fmt = c.attrib["format"] if "format" in c.attrib else "ascii"
 
+        data_type = c.attrib["type"]
+        try:
+            dtype = vtu_to_numpy_type[data_type]
+        except KeyError:
+            raise ReadError(f"Illegal data type '{data_type}'.")
+
         if fmt == "ascii":
             # ascii
-            data = numpy.fromstring(
-                c.text, dtype=vtu_to_numpy_type[c.attrib["type"]], sep=" "
-            )
+            data = np.fromstring(c.text, dtype=dtype, sep=" ")
         elif fmt == "binary":
             reader = (
                 self.read_uncompressed_binary
                 if self.compression is None
                 else self.read_compressed_binary
             )
-            data = reader(c.text.strip(), c.attrib["type"])
+            data = reader(c.text.strip(), dtype)
         elif fmt == "appended":
             offset = int(c.attrib["offset"])
             reader = (
@@ -492,9 +625,9 @@ class VtuReader:
                 if self.compression is None
                 else self.read_compressed_binary
             )
-            data = reader(self.appended_data[offset:], c.attrib["type"])
+            data = reader(self.appended_data[offset:], dtype)
         else:
-            raise ReadError("Unknown data format '{}'.".format(fmt))
+            raise ReadError(f"Unknown data format '{fmt}'.")
 
         if "NumberOfComponents" in c.attrib:
             data = data.reshape(-1, int(c.attrib["NumberOfComponents"]))
@@ -525,6 +658,22 @@ def write(filename, mesh, binary=True, compression="zlib", header_type=None):
     # very memory hungry. See <https://stackoverflow.com/q/59272477/353337>.
     from .._cxml import etree as ET
 
+    # Check if the mesh contains polyhedral cells, this will require special treatment
+    # in certain places.
+    is_polyhedron_grid = False
+    for c in mesh.cells:
+        if c.type[:10] == "polyhedron":
+            is_polyhedron_grid = True
+            break
+    # The current implementation cannot mix polyhedral cells with other cell types.
+    # To write such meshes, represent all cells as polyhedra.
+    if is_polyhedron_grid:
+        for c in mesh.cells:
+            if c.type[:10] != "polyhedron":
+                raise ValueError(
+                    "VTU export cannot mix polyhedral cells with other cell types"
+                )
+
     if not binary:
         logging.warning("VTU ASCII files are only meant for debugging.")
 
@@ -533,8 +682,8 @@ def write(filename, mesh, binary=True, compression="zlib", header_type=None):
             "VTU requires 3D points, but 2D points given. "
             "Appending 0 third component."
         )
-        mesh.points = numpy.column_stack(
-            [mesh.points[:, 0], mesh.points[:, 1], numpy.zeros(mesh.points.shape[0])]
+        mesh.points = np.column_stack(
+            [mesh.points[:, 0], mesh.points[:, 1], np.zeros(mesh.points.shape[0])]
         )
 
     vtk_file = ET.Element(
@@ -548,6 +697,7 @@ def write(filename, mesh, binary=True, compression="zlib", header_type=None):
     header_type = (
         "UInt32" if header_type is None else vtk_file.set("header_type", header_type)
     )
+    assert header_type is not None
 
     if binary and compression:
         # TODO lz4, lzma <https://vtk.org/doc/nightly/html/classvtkDataCompressor.html>
@@ -563,11 +713,25 @@ def write(filename, mesh, binary=True, compression="zlib", header_type=None):
     # <https://github.com/numpy/numpy/issues/10372>.
     points = mesh.points.astype(mesh.points.dtype.newbyteorder("="), copy=False)
     for k, (cell_type, data) in enumerate(mesh.cells):
-        mesh.cells[k] = CellBlock(
-            cell_type, data.astype(data.dtype.newbyteorder("="), copy=False)
-        )
+        # Treatment of polyhedra is different from other types
+        if is_polyhedron_grid:
+            new_cell_info = []
+            for cell_info in data:
+                new_face_info = []
+                for face_info in cell_info:
+                    face_info = np.asarray(face_info)
+                    new_face_info.append(
+                        face_info.astype(face_info.dtype.newbyteorder("="), copy=False)
+                    )
+                new_cell_info.append(new_face_info)
+            mesh.cells[k] = CellBlock(cell_type, new_cell_info)
+        else:
+            mesh.cells[k] = CellBlock(
+                cell_type, data.astype(data.dtype.newbyteorder("="), copy=False)
+            )
     for key, data in mesh.point_data.items():
         mesh.point_data[key] = data.astype(data.dtype.newbyteorder("="), copy=False)
+
     for data in mesh.cell_data.values():
         for k, dat in enumerate(data):
             data[k] = dat.astype(dat.dtype.newbyteorder("="), copy=False)
@@ -606,7 +770,7 @@ def write(filename, mesh, binary=True, compression="zlib", header_type=None):
                     ]
 
                     # collect header
-                    header = numpy.array(
+                    header = np.array(
                         [num_blocks, max_block_size, last_block_size]
                         + [len(b) for b in compressed_blocks],
                         dtype=vtu_to_numpy_type[header_type],
@@ -619,7 +783,7 @@ def write(filename, mesh, binary=True, compression="zlib", header_type=None):
                 def text_writer(f):
                     data_bytes = data.tobytes()
                     # collect header
-                    header = numpy.array(
+                    header = np.array(
                         len(data_bytes), dtype=vtu_to_numpy_type[header_type]
                     )
                     f.write(base64.b64encode(header.tobytes() + data_bytes).decode())
@@ -630,7 +794,7 @@ def write(filename, mesh, binary=True, compression="zlib", header_type=None):
             def text_writer(f):
                 # This write() loop is the bottleneck for the write. Alternatives:
                 # savetxt is super slow:
-                #   numpy.savetxt(f, data.reshape(-1), fmt=fmt)
+                #   np.savetxt(f, data.reshape(-1), fmt=fmt)
                 # joining and writing is a bit faster, but consumes huge amounts of
                 # memory:
                 #   f.write("\n".join(map(fmt.format, data.reshape(-1))))
@@ -638,9 +802,36 @@ def write(filename, mesh, binary=True, compression="zlib", header_type=None):
                     f.write((fmt + "\n").format(item))
 
         da.text_writer = text_writer
-        return
 
-    comment = ET.Comment("This file was created by meshio v{}".format(__version__))
+    def _polyhedron_face_cells(face_cells):
+        # Define the faces of each cell on the format specfied for VTU Polyhedron cells.
+        # These are defined in Mesh.polyhedron_faces, as block data. The block consists
+        # of a nested list (outer list represents cell, inner is faces for this cells),
+        # where the items of the inner list are the nodes of specific faces.
+        #
+        # The output format is specified at https://vtk.org/Wiki/VTK/Polyhedron_Support
+
+        # Initialize array for size of data per cell.
+        data_size_per_cell = np.zeros(len(face_cells), dtype=int)
+
+        # The data itself is of unknown size, and cannot be initialized
+        data = []
+        for ci, cell in enumerate(face_cells):
+            # Number of faces for this cell
+            data.append(len(cell))
+            for face in cell:
+                # Number of nodes for this face
+                data.append(face.size)
+                # The nodes themselves
+                data += face.tolist()
+
+            data_size_per_cell[ci] = len(data)
+
+        # The returned data corresponds to the faces and faceoffsets fields in the
+        # vtu polyhedron data format
+        return data, data_size_per_cell.tolist()
+
+    comment = ET.Comment(f"This file was created by meshio v{__version__}")
     vtk_file.insert(1, comment)
 
     grid = ET.SubElement(vtk_file, "UnstructuredGrid")
@@ -650,7 +841,7 @@ def write(filename, mesh, binary=True, compression="zlib", header_type=None):
         grid,
         "Piece",
         NumberOfPoints="{}".format(len(points)),
-        NumberOfCells="{}".format(total_num_cells),
+        NumberOfCells=f"{total_num_cells}",
     )
 
     # points
@@ -661,47 +852,109 @@ def write(filename, mesh, binary=True, compression="zlib", header_type=None):
     if mesh.cells is not None:
         cls = ET.SubElement(piece, "Cells")
 
-        # create connectivity, offset, type arrays
-        connectivity = numpy.concatenate(
-            [
-                v.data[:, _meshio_to_vtk_order(v.type, v.data.shape[1])].reshape(-1)
+        faces = None
+        faceoffsets = None
+
+        if is_polyhedron_grid:
+            # The VTK polyhedron format requires both Cell-node connectivity, and a
+            # definition of faces. The cell-node relation must be recoved from the
+            # cell-face-nodes currently in CellBlocks.
+            # NOTE: If polyhedral cells are implemented for more mesh types, this code
+            # block may be useful for those as well.
+            con = []
+            num_nodes_per_cell = []
+            for block in mesh.cells:
+                for cell in block.data:
+                    nodes_this_cell = []
+                    for face in cell:
+                        nodes_this_cell += face.tolist()
+                    unique_nodes = np.unique(nodes_this_cell).tolist()
+
+                    con += unique_nodes
+                    num_nodes_per_cell.append(len(unique_nodes))
+
+            connectivity = np.array(con)
+            # offsets = np.hstack(([0], np.cumsum(num_nodes_per_cell)[:-1]))
+            offsets = np.cumsum(num_nodes_per_cell)
+
+            # Initialize data structures for polyhedral cells
+            faces = []
+            faceoffsets = []
+
+        else:
+            # create connectivity, offset, type arrays
+            connectivity = np.concatenate(
+                [
+                    v.data[:, _meshio_to_vtk_order(v.type, v.data.shape[1])].reshape(-1)
+                    for v in mesh.cells
+                ]
+            )
+
+            # offset (points to the first element of the next cell)
+            offsets = [
+                v.data.shape[1]
+                * np.arange(1, v.data.shape[0] + 1, dtype=connectivity.dtype)
                 for v in mesh.cells
             ]
-        )
-
-        # offset (points to the first element of the next cell)
-        offsets = [
-            v.data.shape[1]
-            * numpy.arange(1, v.data.shape[0] + 1, dtype=connectivity.dtype)
-            for v in mesh.cells
-        ]
-        for k in range(1, len(offsets)):
-            offsets[k] += offsets[k - 1][-1]
-        offsets = numpy.concatenate(offsets)
+            for k in range(1, len(offsets)):
+                offsets[k] += offsets[k - 1][-1]
+            offsets = np.concatenate(offsets)
 
         # types
         types_array = []
         for k, v in mesh.cells:
             # For polygon and polyhedron grids, the number of nodes is part of the cell
             # type key. This part must be stripped away.
-            if k[:7] == "polygon":
-                key_ = k[:7]
-            elif k[:10] == "polyhedron":
-                key_ = k[:10]
-            else:
+            special_cells = [
+                "polygon",
+                "polyhedron",
+                "VTK_LAGRANGE_CURVE",
+                "VTK_LAGRANGE_TRIANGLE",
+                "VTK_LAGRANGE_QUADRILATERAL",
+                "VTK_LAGRANGE_TETRAHEDRON",
+                "VTK_LAGRANGE_HEXAHEDRON",
+                "VTK_LAGRANGE_WEDGE",
+                "VTK_LAGRANGE_PYRAMID",
+            ]
+            key_ = None
+            for string in special_cells:
+                if k.startswith(string):
+                    key_ = string
+
+            if key_ is None:
                 # No special treatment
                 key_ = k
-            types_array.append(numpy.full(len(v), meshio_to_vtk_type[key_]))
 
-        types = numpy.concatenate(
+            # further adaptions for polyhedron
+            if k.startswith("polyhedron"):
+                # Get face-cell relation on the vtu format. See comments in helper
+                # function for more information of how to specify this.
+                faces_loc, faceoffsets_loc = _polyhedron_face_cells(v)
+                # Adjust offsets to global numbering
+                assert faceoffsets is not None
+                if len(faceoffsets) > 0:
+                    faceoffsets_loc = [fi + faceoffsets[-1] for fi in faceoffsets_loc]
+
+                assert faces is not None
+                faces += faces_loc
+                faceoffsets += faceoffsets_loc
+
+            types_array.append(np.full(len(v), meshio_to_vtk_type[key_]))
+
+        types = np.concatenate(
             types_array
-            # [numpy.full(len(v), meshio_to_vtk_type[k]) for k, v in mesh.cells]
+            # [np.full(len(v), meshio_to_vtk_type[k]) for k, v in mesh.cells]
         )
 
         numpy_to_xml_array(cls, "connectivity", connectivity)
         numpy_to_xml_array(cls, "offsets", offsets)
         numpy_to_xml_array(cls, "types", types)
 
+        if is_polyhedron_grid:
+            # Also store face-node relation
+            numpy_to_xml_array(cls, "faces", np.array(faces, dtype=int))
+            numpy_to_xml_array(cls, "faceoffsets", np.array(faceoffsets, dtype=int))
+
     if mesh.point_data:
         pd = ET.SubElement(piece, "PointData")
         for name, data in mesh.point_data.items():
