@@ -3,10 +3,11 @@ I/O for XDMF.
 http://www.xdmf.org/index.php/XDMF_Model_and_Format
 """
 import os
+import pathlib
 from io import BytesIO
 from xml.etree import ElementTree as ET
 
-import numpy
+import numpy as np
 
 from .._common import cell_data_from_raw, raw_from_cell_data, write_xml
 from .._exceptions import ReadError, WriteError
@@ -46,7 +47,7 @@ class XdmfReader:
             return self.read_xdmf2(root)
 
         if version.split(".")[0] != "3":
-            raise ReadError("Unknown XDMF version {}.".format(version))
+            raise ReadError(f"Unknown XDMF version {version}.")
 
         return self.read_xdmf3(root)
 
@@ -60,7 +61,7 @@ class XdmfReader:
                 return self._read_data_item(
                     root.find(".//" + "/".join(xpath.split("/")[2:])), root
                 )
-            raise ValueError("Can't read XPath {}.".format(xpath))
+            raise ValueError(f"Can't read XPath {xpath}.")
 
         dims = [int(d) for d in data_item.get("Dimensions").split()]
 
@@ -85,23 +86,25 @@ class XdmfReader:
             precision = "4"
 
         if data_item.get("Format") == "XML":
-            return numpy.fromstring(
+            return np.fromstring(
                 data_item.text,
                 dtype=xdmf_to_numpy_type[(data_type, precision)],
                 sep=" ",
             ).reshape(dims)
         elif data_item.get("Format") == "Binary":
-            return numpy.fromfile(
+            return np.fromfile(
                 data_item.text.strip(), dtype=xdmf_to_numpy_type[(data_type, precision)]
             ).reshape(dims)
         elif data_item.get("Format") != "HDF":
-            raise ReadError("Unknown XDMF Format '{}'.".format(data_item.get("Format")))
+            fmt = data_item.get("Format")
+            raise ReadError(f"Unknown XDMF Format '{fmt}'.")
 
         info = data_item.text.strip()
         filename, h5path = info.split(":")
 
         # The HDF5 file path is given with respect to the XDMF (XML) file.
-        full_hdf5_path = os.path.join(os.path.dirname(self.filename), filename)
+        dirname = pathlib.Path(self.filename).resolve().parent
+        full_hdf5_path = dirname / filename
 
         f = h5py.File(full_hdf5_path, "r")
 
@@ -111,7 +114,7 @@ class XdmfReader:
 
         for key in h5path.split("/"):
             f = f[key]
-        # `[()]` gives a numpy.ndarray
+        # `[()]` gives a np.ndarray
         return f[()]
 
     def read_information(self, c_data):
@@ -119,9 +122,14 @@ class XdmfReader:
         root = ET.fromstring(c_data)
         for child in root:
             str_tag = child.get("key")
-            dim = int(child.get("dim"))
+            dim = child.get("dim")
+            if dim is None:
+                raise ReadError()
+            dim = int(dim)
+            if child.text is None:
+                raise ReadError()
             num_tag = int(child.text)
-            field_data[str_tag] = numpy.array([num_tag, dim])
+            field_data[str_tag] = np.array([num_tag, dim])
         return field_data
 
     def read_xdmf2(self, root):  # noqa: C901
@@ -156,7 +164,7 @@ class XdmfReader:
                 topology_type = c.get("TopologyType")
                 if topology_type == "Mixed":
                     cells = translate_mixed_cells(
-                        numpy.fromstring(
+                        np.fromstring(
                             data_items[0].text,
                             int,
                             int(data_items[0].get("Dimensions")),
@@ -201,7 +209,7 @@ class XdmfReader:
                     if c.get("Center") != "Grid":
                         raise ReadError()
             else:
-                raise ReadError("Unknown section '{}'.".format(c.tag))
+                raise ReadError(f"Unknown section '{c.tag}'.")
 
         cell_data = cell_data_from_raw(cells, cell_data_raw)
 
@@ -266,7 +274,7 @@ class XdmfReader:
                     geometry_type = c.get("GeometryType")
 
                 if geometry_type not in ["XY", "XYZ"]:
-                    raise ReadError('Illegal geometry type "{}".'.format(geometry_type))
+                    raise ReadError(f'Illegal geometry type "{geometry_type}".')
 
                 data_items = list(c)
                 if len(data_items) != 1:
@@ -300,7 +308,7 @@ class XdmfReader:
                         raise ReadError()
                     cell_data_raw[name] = data
             else:
-                raise ReadError("Unknown section '{}'.".format(c.tag))
+                raise ReadError(f"Unknown section '{c.tag}'.")
 
         cell_data = cell_data_from_raw(cells, cell_data_raw)
 
@@ -322,7 +330,7 @@ class XdmfWriter:
         if data_format not in ["XML", "Binary", "HDF"]:
             raise WriteError(
                 "Unknown XDMF data format "
-                "'{}' (use 'XML', 'Binary', or 'HDF'.)".format(data_format)
+                f"'{data_format}' (use 'XML', 'Binary', or 'HDF'.)"
             )
 
         self.filename = filename
@@ -343,11 +351,11 @@ class XdmfWriter:
         #     grid, "Information", Name="Information", Value=str(len(mesh.field_data))
         # )
 
-        self.points(grid, mesh.points)
+        self.write_points(grid, mesh.points)
         # self.field_data(mesh.field_data, information)
-        self.cells(mesh.cells, grid)
-        self.point_data(mesh.point_data, grid)
-        self.cell_data(mesh.cell_data, grid)
+        self.write_cells(mesh.cells, grid)
+        self.write_point_data(mesh.point_data, grid)
+        self.write_cell_data(mesh.cell_data, grid)
 
         ET.register_namespace("xi", "https://www.w3.org/2001/XInclude/")
 
@@ -357,12 +365,11 @@ class XdmfWriter:
         if self.data_format == "XML":
             s = BytesIO()
             fmt = dtype_to_format_string[data.dtype.name]
-            numpy.savetxt(s, data, fmt)
+            np.savetxt(s, data, fmt)
             return "\n" + s.getvalue().decode()
         elif self.data_format == "Binary":
-            bin_filename = "{}{}.bin".format(
-                os.path.splitext(self.filename)[0], self.data_counter
-            )
+            base = os.path.splitext(self.filename)[0]
+            bin_filename = f"{base}{self.data_counter}.bin"
             self.data_counter += 1
             # write binary data to file
             with open(bin_filename, "wb") as f:
@@ -370,8 +377,8 @@ class XdmfWriter:
             return bin_filename
 
         if self.data_format != "HDF":
-            raise WriteError('Unknown data format "{}"'.format(self.data_format))
-        name = "data{}".format(self.data_counter)
+            raise WriteError(f'Unknown data format "{self.data_format}"')
+        name = f"data{self.data_counter}"
         self.data_counter += 1
         self.h5_file.create_dataset(
             name,
@@ -381,15 +388,11 @@ class XdmfWriter:
         )
         return os.path.basename(self.h5_filename) + ":/" + name
 
-    def points(self, grid, points):
-        if points.shape[1] == 1:
-            geometry_type = "X"
-        elif points.shape[1] == 2:
-            geometry_type = "XY"
-        else:
-            if points.shape[1] != 3:
-                raise WriteError()
-            geometry_type = "XYZ"
+    def write_points(self, grid, points):
+        if points.shape[1] > 3:
+            raise WriteError("Can only write points up to dimension 3.")
+
+        geometry_type = "XYZ"[: points.shape[1]]
 
         geo = ET.SubElement(grid, "Geometry", GeometryType=geometry_type)
         dt, prec = numpy_to_xdmf_dtype[points.dtype.name]
@@ -404,7 +407,7 @@ class XdmfWriter:
         )
         data_item.text = self.numpy_to_xml_string(points)
 
-    def cells(self, cells, grid):
+    def write_cells(self, cells, grid):
         if len(cells) == 1:
             meshio_type = cells[0].type
             num_cells = len(cells[0].data)
@@ -437,16 +440,16 @@ class XdmfWriter:
                 TopologyType="Mixed",
                 NumberOfElements=str(total_num_cells),
             )
-            total_num_cell_items = sum(numpy.prod(c.data.shape) for c in cells)
+            total_num_cell_items = sum(np.prod(c.data.shape) for c in cells)
             num_vertices_and_lines = sum(
                 c.data.shape[0] for c in cells if c.type in {"vertex", "line"}
             )
             dim = str(total_num_cell_items + total_num_cells + num_vertices_and_lines)
-            cd = numpy.concatenate(
+            cd = np.concatenate(
                 [
-                    numpy.hstack(
+                    np.hstack(
                         [
-                            numpy.full(
+                            np.full(
                                 (value.shape[0], 2 if key in {"vertex", "line"} else 1),
                                 meshio_type_to_xdmf_index[key],
                             ),
@@ -467,7 +470,7 @@ class XdmfWriter:
             )
             data_item.text = self.numpy_to_xml_string(cd)
 
-    def point_data(self, point_data, grid):
+    def write_point_data(self, point_data, grid):
         for name, data in point_data.items():
             att = ET.SubElement(
                 grid,
@@ -488,7 +491,7 @@ class XdmfWriter:
             )
             data_item.text = self.numpy_to_xml_string(data)
 
-    def cell_data(self, cell_data, grid):
+    def write_cell_data(self, cell_data, grid):
         raw = raw_from_cell_data(cell_data)
         for name, data in raw.items():
             att = ET.SubElement(
@@ -530,5 +533,8 @@ def write(*args, **kwargs):
 
 # TODO register all xdmf except hdf outside this try block
 register(
-    "xdmf", [".xdmf", ".xmf"], read, {"xdmf": write},
+    "xdmf",
+    [".xdmf", ".xmf"],
+    read,
+    {"xdmf": write},
 )
