@@ -1,7 +1,7 @@
 import pathlib
 
 import helpers
-import numpy
+import numpy as np
 import pytest
 
 import meshio
@@ -23,7 +23,7 @@ import meshio
     ],
 )
 def test(mesh):
-    helpers.write_read(meshio.nastran.write, meshio.nastran.read, mesh, 1.0e-15)
+    helpers.write_read(meshio.nastran.write, meshio.nastran.read, mesh, 1.0e-13)
 
 
 @pytest.mark.parametrize("filename", ["cylinder.fem", "cylinder_cells_first.fem"])
@@ -34,7 +34,7 @@ def test_reference_file(filename):
     mesh = meshio.read(filename)
 
     # points
-    assert numpy.isclose(mesh.points.sum(), 16.5316866)
+    assert np.isclose(mesh.points.sum(), 16.5316866)
 
     # cells
     ref_num_cells = {
