@@ -1,9 +1,9 @@
-import os
-
-import numpy
-import pytest
+import pathlib
 
 import helpers
+import numpy as np
+import pytest
+
 import meshio
 
 
@@ -18,10 +18,11 @@ def test_neuroglancer(mesh):
 
 @pytest.mark.parametrize("filename, ref_sum, ref_num_cells", [("simple1", 20, 4)])
 def test_reference_file(filename, ref_sum, ref_num_cells):
-    this_dir = os.path.dirname(os.path.abspath(__file__))
-    filename = os.path.join(this_dir, "meshes", "neuroglancer", filename)
+    this_dir = pathlib.Path(__file__).resolve().parent
+    filename = this_dir / "meshes" / "neuroglancer" / filename
+
     mesh = meshio.read(filename, "neuroglancer")
     tol = 1.0e-5
-    s = numpy.sum(mesh.points)
+    s = np.sum(mesh.points)
     assert abs(s - ref_sum) < tol * abs(ref_sum)
     assert len(mesh.cells[0].data) == ref_num_cells
