@@ -1,7 +1,7 @@
 import pathlib
 
 import helpers
-import numpy
+import numpy as np
 import pytest
 
 import meshio
@@ -15,7 +15,7 @@ def test_obj(mesh):
         return meshio.obj.write(*args, **kwargs)
 
     for k, c in enumerate(mesh.cells):
-        mesh.cells[k] = meshio.CellBlock(c.type, c.data.astype(numpy.int32))
+        mesh.cells[k] = meshio.CellBlock(c.type, c.data.astype(np.int32))
 
     helpers.write_read(writer, meshio.obj.read, mesh, 1.0e-12)
 
@@ -29,7 +29,7 @@ def test_reference_file(filename, ref_sum, ref_num_cells):
 
     mesh = meshio.read(filename)
     tol = 1.0e-5
-    s = numpy.sum(mesh.points)
+    s = np.sum(mesh.points)
     assert abs(s - ref_sum) < tol * abs(ref_sum)
     assert mesh.cells[0].type == "triangle"
     assert len(mesh.cells[0].data) == ref_num_cells
