@@ -1,7 +1,7 @@
-import os
+import pathlib
 
 import helpers
-import numpy
+import numpy as np
 import pytest
 
 import meshio
@@ -11,6 +11,7 @@ test_set = [
     helpers.tet_mesh,
     helpers.hex_mesh,
 ]
+this_dir = pathlib.Path(__file__).resolve().parent
 
 
 @pytest.mark.parametrize("mesh", test_set)
@@ -28,18 +29,17 @@ def test(mesh):
 def test_structured(
     filename, ref_num_cells, ref_num_points, ref_num_unique_tags, sum_tags
 ):
-    this_dir = os.path.dirname(os.path.abspath(__file__))
-    filename = os.path.join(this_dir, "meshes", "su2", filename)
+    filename = this_dir / "meshes" / "su2" / filename
 
     mesh = meshio.read(filename)
 
     assert sum(len(block.data) for block in mesh.cells) == ref_num_cells
     assert len(mesh.points) == ref_num_points
 
-    all_tags = numpy.concatenate([tags for tags in mesh.cell_data["su2:tag"]])
+    all_tags = np.concatenate([tags for tags in mesh.cell_data["su2:tag"]])
 
     assert sum(all_tags) == sum_tags
 
-    all_unique_tags = numpy.unique(all_tags)
+    all_unique_tags = np.unique(all_tags)
 
     assert len(all_unique_tags) == ref_num_unique_tags + 1
