@@ -2,7 +2,7 @@ import pathlib
 from functools import partial
 
 import helpers
-import numpy
+import numpy as np
 import pytest
 
 import meshio
@@ -20,6 +20,8 @@ test_set = [
     helpers.hex_mesh,
     helpers.hex20_mesh,
     helpers.polygon_mesh,
+    helpers.pyramid_mesh,
+    helpers.wedge_mesh,
     helpers.add_point_data(helpers.tri_mesh, 1),
     helpers.add_point_data(helpers.tri_mesh, 2),
     helpers.add_point_data(helpers.tri_mesh, 3),
@@ -58,7 +60,7 @@ def test_reference_file(filename, ref_sum, ref_num_cells, binary):
 
     mesh = meshio.read(filename)
     tol = 1.0e-2
-    s = numpy.sum(mesh.points)
+    s = np.sum(mesh.points)
     assert abs(s - ref_sum) < tol * ref_sum
     assert mesh.cells[0].type == "triangle"
     assert len(mesh.cells[0].data) == ref_num_cells
@@ -75,6 +77,8 @@ def test_reference_file(filename, ref_sum, ref_num_cells, binary):
         ("03_rectilinear.vtk", "hexahedron", 72, 147),
         ("04_rectilinear.vtk", "quad", 27, 40),
         ("05_rectilinear.vtk", "quad", 27, 40),
+        ("06_unstructured.vtk", "hexahedron", 12, 42),
+        ("gh-935.vtk", "triangle", 2, 6),
     ],
 )
 def test_structured(filename, ref_cells, ref_num_cells, ref_num_pnt):
@@ -93,5 +97,17 @@ def test_pathlike():
     meshio.read(this_dir / "meshes" / "vtk" / "rbc_001.vtk")
 
 
+@pytest.mark.parametrize(
+    "filename, ref_num_points, ref_num_cells", [("06_color_scalars.vtk", 5, 2)]
+)
+def test_color_scalars(filename, ref_num_points, ref_num_cells):
+    this_dir = pathlib.Path(__file__).resolve().parent
+    filename = this_dir / "meshes" / "vtk" / filename
+
+    mesh = meshio.read(filename)
+    assert len(mesh.points) == ref_num_points
+    assert len(mesh.cells) == ref_num_cells
+
+
 if __name__ == "__main__":
     test(helpers.tri_mesh, binary=True)
