@@ -1,9 +1,9 @@
-import os
-
-import numpy
-import pytest
+import pathlib
 
 import helpers
+import numpy as np
+import pytest
+
 import meshio
 
 
@@ -20,11 +20,12 @@ def test_wkt(mesh):
     [("simple.wkt", 4, 2), ("whitespaced.wkt", 3.2, 2)],
 )
 def test_reference_file(filename, ref_sum, ref_num_cells):
-    this_dir = os.path.dirname(os.path.abspath(__file__))
-    filename = os.path.join(this_dir, "meshes", "wkt", filename)
+    this_dir = pathlib.Path(__file__).resolve().parent
+    filename = this_dir / "meshes" / "wkt" / filename
+
     mesh = meshio.read(filename)
     tol = 1.0e-5
-    s = numpy.sum(mesh.points)
+    s = np.sum(mesh.points)
     assert abs(s - ref_sum) < tol * abs(ref_sum)
     assert mesh.cells[0].type == "triangle"
     assert len(mesh.cells[0].data) == ref_num_cells
